package com.galaxis.manatee.dao;

import com.galaxis.capsule.util.GalaxisRepository;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunProjectCostDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunWeekMemberHourDO;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Repository
public interface ChuanyunWeekMemberHourDao extends GalaxisRepository<ChuanyunWeekMemberHourDO,String> {

    /**
     * 根据状态删除数据
     * @param status    状态
     * @return  删除条数
     */
    @Transactional
    Integer deleteAllByStatus(String status);

    /**
     * 人员-项目-周度工时汇总
     * @return
     */
    @Query(value="SELECT chuanyun_self_work_hour.object_id," +
            "DATE_FORMAT(day_log_date,'%Y') as year," +
            "WEEKOFYEAR(chuanyun_self_work_hour.day_log_date) AS week," +
            "project_id,user_id," +
            "SUM(standard_work_hour)AS standard_work_hour," +
            "project_type," +
            "`status`" +
            "FROM chuanyun_self_work_hour " +
            "WHERE project_id !='' " +
            "GROUP BY chuanyun_self_work_hour.project_id," +
            "chuanyun_self_work_hour.user_id," +
            "DATE_FORMAT(day_log_date,'%Y')," +
            "WEEKOFYEAR(chuanyun_self_work_hour.day_log_date)," +
            "chuanyun_self_work_hour.`status`",nativeQuery = true)
    List<ChuanyunWeekMemberHourDO> getWeekMemberHour();

    /**
     * 项目人员周度是否存在
     * @param projectId
     * @param userId
     * @param week
     * @return
     */
    ChuanyunWeekMemberHourDO findByProjectIdAndUserIdAndWeekAndStatusAndProjectTypeAndYear(String projectId,String userId,String week,String status,String projectType,String year);
}
